package com.lj.Sirui;

import java.io.ByteArrayOutputStream;
import java.io.UnsupportedEncodingException;
import java.security.Key;
import java.security.KeyFactory;
import java.security.MessageDigest;
import java.security.spec.X509EncodedKeySpec;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.crypto.Cipher;

import org.apache.commons.codec.binary.Base64;

public class RSAUtils
{
    /**
     * RSA最大加密明文大小
     */
    private static final int MAX_ENCRYPT_BLOCK = 117;

    /**
     * RSA最大解密密文大小
     */
    private static final int MAX_DECRYPT_BLOCK = 128;

    /**
     * 加密算法RSA
     */
    public static final String KEY_ALGORITHM = "RSA";

    public static final String RSA_PADDING = "RSA/ECB/PKCS1Padding";

    public static final String CHARSET = "UTF-8";

    /**
     * 用公钥对整体请求参数加密方法
     * @param data [JSON格式的接口请求参数]
     * @param publicKey [公钥]
     * @param publicKey [公钥]
     * @return
     * @throws Exception
     */
    public static String encryptByPublicKeyEx(String data, String publicKey) throws Exception
    {
        byte[] keyBytes = Base64.decodeBase64(publicKey);
        X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(keyBytes);
        KeyFactory keyFactory = KeyFactory.getInstance(KEY_ALGORITHM);
        Key publicK = keyFactory.generatePublic(x509KeySpec);
        // 对数据加密
        Cipher cipher = Cipher.getInstance(RSA_PADDING);
        cipher.init(Cipher.ENCRYPT_MODE, publicK);
        List<String> datas = parse2StringLessThan117Bytes(data);
        StringBuffer buffer = new StringBuffer();
        byte[] cache = null;
        // 对数据分段加密, 并使用逗号分开
        for (int i = 0; i < datas.size(); i++)
        {
            String temp = datas.get(i);
            byte[] dataCache = temp.getBytes(CHARSET);
            cache = cipher.doFinal(dataCache, 0, dataCache.length);
            buffer.append(Base64.encodeBase64String(cache));
            if (i != datas.size() - 1)
            {
                buffer.append(",");
            }
        }
        return buffer.toString();
    }

    private static List<String> parse2StringLessThan117Bytes(String data) throws UnsupportedEncodingException
    {
        List<String> datas = new ArrayList<String>();
        char[] cDatas = data.toCharArray();
        StringBuffer sbuffer = new StringBuffer();
        for (int i = 0; i < cDatas.length; i++)
        {
            char c = cDatas[i];
            String s = String.valueOf(c);
            byte[] b = s.getBytes(CHARSET);
            int bLen = b.length;
            if ((sbuffer.toString().getBytes(CHARSET).length + bLen) <= MAX_ENCRYPT_BLOCK)
            {
                sbuffer.append(s);
                if ((sbuffer.toString().getBytes(CHARSET).length) == MAX_ENCRYPT_BLOCK)
                {
                    // 加了bLen刚好等于117
                    datas.add(sbuffer.toString());
                    sbuffer = new StringBuffer();
                }
            } else if ((sbuffer.toString().getBytes(CHARSET).length + bLen) > MAX_ENCRYPT_BLOCK)
            {
                // sbuffer的byte的长度+bLen大于117
                datas.add(sbuffer.toString());
                sbuffer = new StringBuffer();
                sbuffer.append(s);
            }
        }
        if (sbuffer.toString().length() != 0)
        {
            // 最后一个字符串添加到容器
            datas.add(sbuffer.toString());
        }
        return datas;
    }

    /**
     * signature签名算法方法
     * @param appSecret [appSecret]
     * @param timestamp [时间戳 ，单位：秒]
     * @param nonce [16位随机数]
     * @param paramsString [JSON格式的接口请求参数]
     * @return
     */
    public static String generateSignWithParamsString(String appSecret, String timestamp, String nonce, String paramsString)
    {
        String[] params = { appSecret, timestamp, nonce, paramsString };
        Arrays.sort(params);
        StringBuffer buffer = new StringBuffer();
        for (int i = 0; i < params.length; i++)
        {
            buffer.append(params[i]);
        }

        try
        {
            MessageDigest digest = MessageDigest.getInstance("SHA-1");
            digest.update(buffer.toString().getBytes(CHARSET));
            // 获取字节数组
            byte messageDigest[] = digest.digest();
            // Create Hex String
            StringBuffer hexString = new StringBuffer();
            // 字节数组转换为十六进制数
            for (int i = 0; i < messageDigest.length; i++)
            {
                String shaHex = Integer.toHexString(messageDigest[i] & 0xFF);
                if (shaHex.length() < 2)
                {
                    hexString.append(0);
                }
                hexString.append(shaHex);
            }
            return hexString.toString();
        } catch (Exception e)
        {
            e.printStackTrace();
        }
        return null;
    }

    
    /**
     * 公钥解密
     * @param encryptedData [接口返回的data数据]
     * @param publicKey [公钥]
     * @return
     * @throws Exception
     */
    public static String decryptByPublicKey(String encryptedData, String publicKey) throws Exception
    {
        byte[] byteEncryptedData=Base64.decodeBase64(encryptedData);
        
        byte[] keyBytes = Base64.decodeBase64(publicKey);
        
        X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(keyBytes);
        KeyFactory keyFactory = KeyFactory.getInstance(KEY_ALGORITHM);
        Key publicK = keyFactory.generatePublic(x509KeySpec);
        Cipher cipher = Cipher.getInstance(RSA_PADDING);
        cipher.init(Cipher.DECRYPT_MODE, publicK);
        int inputLen = byteEncryptedData.length;
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        int offSet = 0;
        byte[] cache;
        int i = 0;
        // 对数据分段解密
        while (inputLen - offSet > 0)
        {
            if (inputLen - offSet > MAX_DECRYPT_BLOCK)
            {
                cache = cipher.doFinal(byteEncryptedData, offSet, MAX_DECRYPT_BLOCK);
            } else
            {
                cache = cipher.doFinal(byteEncryptedData, offSet, inputLen - offSet);
            }
            out.write(cache, 0, cache.length);
            i++;
            offSet = i * MAX_DECRYPT_BLOCK;
        }
        byte[] decryptedData = out.toByteArray();
        String decryptedDataStr=new String(decryptedData);
        out.close();
        return decryptedDataStr;
    }
}
