package com.lj.Socket;

import javax.websocket.*;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by john on 2018/12/10.
 */
@ServerEndpoint("/websocketchat")
public class WebSocketChat {
    //存储历史数据
    private static Map<String, String> historyData = new HashMap<String, String>();
    //静态变量，用来记录当前在线连接数。应该把它设计成线程安全的。
    private static int onlineCount = 0;
    private static Map<String, WebSocketChat> socketMap = new HashMap<String, WebSocketChat>();

    //与某个客户端的连接会话，需要通过它来给客户端发送数据
    private Session session;

    /**
     * 连接建立成功调用的方法
     *
     * @param session 可选的参数。session为与某个客户端的连接会话，需要通过它来给客户端发送数据
     */
    @OnOpen
    public void onOpen(Session session) {
        this.session = session;
        socketMap.put(this.session.getQueryString(), this);
        addOnlineCount();
        System.out.println("" + this.session.getQueryString() + "加入！Chat当前在线人数为：" + getOnlineCount());
    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose() {
        socketMap.remove(this.session.getQueryString());
        subOnlineCount();           //在线数减1
        System.out.println("有一连接关闭！Chat当前在线人数为：" + getOnlineCount());
    }

    /**
     * 收到客户端消息后调用的方法
     *
     * @param message 客户端发送过来的消息
     * @param session 可选的参数
     */
    @OnMessage
    public void onMessage(String message, Session session) {
        //fromuserid,touserid,mess
        String[] sendInfo = message.split(",");
        WebSocketChat item = socketMap.get("userid=" + sendInfo[1]);
        try {
            if (item != null) {
                item.sendMessage(sendInfo[2]);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 发生错误时调用
     *
     * @param session
     * @param error
     */
    @OnError
    public void onError(Session session, Throwable error) {
        System.out.println("发生错误");
        error.printStackTrace();
    }


    /**
     * 这个方法与上面几个方法不一样。没有用注解，是根据自己需要添加的方法。
     *
     * @param message
     * @throws IOException
     */
    public void sendMessage(String message) throws IOException {
        this.session.getBasicRemote().sendText(message);
    }

    public static synchronized int getOnlineCount() {
        return onlineCount;
    }

    public static synchronized void addOnlineCount() {
        WebSocketChat.onlineCount++;
    }

    public static synchronized void subOnlineCount() {
        WebSocketChat.onlineCount--;
    }
}
