package com.lj.Socket;
import com.lj.others.*;
import net.sf.json.JSONObject;

import javax.websocket.*;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.*;
/**
 * @ServerEndpoint 注解是一个类层次的注解，它的功能主要是将目前的类定义成一个websocket服务器端,
 * 注解的值将被用于监听用户连接的终端访问URL地址,客户端可以通过这个URL来连接到WebSocket服务器端
 */
@ServerEndpoint("/websocket")
public class WebSocketTest {

    //存储历史数据
    private static Map<String, String> historyData = new HashMap<String, String>();
    //静态变量，用来记录当前在线连接数。应该把它设计成线程安全的。
    private static int onlineCount = 0;

    //concurrent包的线程安全Set，用来存放每个客户端对应的MyWebSocket对象。若要实现服务端与单一客户端通信的话，可以使用Map来存放，其中Key可以为用户标识
    //private static  CopyOnWriteArraySet<WebSocketTest> webSocketSet = new CopyOnWriteArraySet<WebSocketTest>();

    private static HashMap<String,WebSocketTest> socketMap=new HashMap<String,WebSocketTest>() ;


    //与某个客户端的连接会话，需要通过它来给客户端发送数据
    private Session session;

    /**
     * 连接建立成功调用的方法
     *
     * @param session 可选的参数。session为与某个客户端的连接会话，需要通过它来给客户端发送数据
     */
    @OnOpen
    public void onOpen(Session session) {
       // this.session = session;
       // webSocketSet.add(this);     //加入set中
        //System.out.println("有新连接加入！当前在线人数为" + getOnlineCount());
        //addOnlineCount();           //在线数加1

        this.session=session;
        socketMap.put(this.session.getQueryString(),this);
        addOnlineCount();
        System.out.println(""+this.session.getQueryString()+"加入！New当前在线人数为：" + getOnlineCount());


        //GetData();
        // Timer timer = new Timer();
        // TimerTask task = new TimerTask() {
        //      @Override
        //       public void run() {
        //    }
        //        GetData();
        // };
        // long delay = 0;
        // long intevalPeriod = 30* 1000;
        // timer.scheduleAtFixedRate(task, delay, intevalPeriod);
    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose() {

        //socketMap.remove(this);
       // webSocketSet.remove(this);  //从set中删除
        subOnlineCount();           //在线数减1
        System.out.println("有一连接关闭！当前在线人数为：" + getOnlineCount());
    }

    /**
     * 收到客户端消息后调用的方法
     *
     * @param message 客户端发送过来的消息
     * @param session 可选的参数
     */
    @OnMessage
    public void onMessage(String message, Session session) {
        //fromuserid,touserid,mess
        String[] sendInfo = message.split(",");
        WebSocketTest item = socketMap.get("userid=" + sendInfo[1]);
        try {
            if(item!=null) {
                item.sendMessage(sendInfo[2]);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }

        //群发消息
      /*  for (WebSocketTest item : webSocketSet) {
            try {
                item.sendMessage("服务器转：" + message);
            } catch (IOException e) {
                e.printStackTrace();
                continue;
            }
        }*/
    }

    /**
     * 发生错误时调用
     *
     * @param session
     * @param error
     */
    @OnError
    public void onError(Session session, Throwable error) {
        System.out.println("发生错误");
        error.printStackTrace();
    }


    /**
     * 这个方法与上面几个方法不一样。没有用注解，是根据自己需要添加的方法。
     *
     * @param message
     * @throws IOException
     */
    public void sendMessage(String message) throws IOException {
        this.session.getBasicRemote().sendText(message);
    }

    public static synchronized int getOnlineCount() {
        return onlineCount;
    }

    public static synchronized void addOnlineCount() {
        WebSocketTest.onlineCount++;
    }

    public static synchronized void subOnlineCount() {
        WebSocketTest.onlineCount--;
    }


    public  void GetData() {
        try {
            SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");//设置日期格式
            String currDate = df.format(new Date());// new Date()为获取当前系统时间

            LJTotal ljTotal = GetTotal();
            CSTotal csTotal = GetCSTotal();
            List<TermOrder> termlist = GetOrderData();
            List<OrderPhase> phaseList = GetPhases();

            OrdersTerm ordersTerm = new OrdersTerm();
            ordersTerm.setDonedate(currDate);
            ordersTerm.setLjtotal(ljTotal);
            ordersTerm.setCstotal(csTotal);

            ordersTerm.setTermList(termlist);
            ordersTerm.setPhaseList(phaseList);
            JSONObject jsonObj = JSONObject.fromObject(ordersTerm);//将java对象转换为json对象
            sendMessage(jsonObj.toString());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static LJTotal GetTotal() {
        LJTotal ljTotal = new LJTotal();
        ResultSet rs = SQLHelper.executeQuery(" USE Mall " +
                "DECLARE @d1 DATETIME,@d2 DATETIME\n" +
                "SET @d1=convert(varchar(100),getdate(),23)+' 00:00:00'\n" +
                "SET @d2=convert(varchar(100),getdate(),23)+' 23:59:59' " +
                " \n" +
                "SELECT ISNULL(SUM(ISNULL(T.[sum],0)/100.0),0) order_sum,COUNT(1) order_count,\n" +
                "ISNULL(SUM(CASE WHEN T.ShippingTime>T.CreationDate THEN 1 ELSE 0 END),0) ship_count\n" +
                "FROM [Mall].[dbo].[Orders] T        \n" +
                "WHERE T.OrderStatus not in(0,100,200) AND T.IsReturn=0\n" +
                "AND T.DeptCode NOT LIKE '000213%' --长沙O2O客服部\n" +
                "AND T.DeptCode NOT LIKE '000218%' --O2O部\n" +
                "AND T.DeptCode NOT LIKE '001701%' --门店电商中心\n" +
                "AND T.OrderTime>=@d1 AND  T.OrderTime<=@d2");
        try {
            while (rs.next()) {
                ljTotal.setOrder_sum(rs.getObject("order_sum").toString());
                ljTotal.setOrder_count(rs.getObject("order_count").toString());
                ljTotal.setShip_count(rs.getObject("ship_count").toString());
                return ljTotal;
            }
        } catch (SQLException e) {
            e.printStackTrace();

        }

        return ljTotal;
    }

    public static  CSTotal GetCSTotal() {
        CSTotal csTotal=new CSTotal();
        ResultSet rs = SQLHelper.executeQuery(" USE Mall " +
                "  DECLARE @d1 DATETIME,@d2 DATETIME\n" +
                "                SET @d1=convert(varchar(100),getdate(),23)+' 00:00:00'\n" +
                "                SET @d2=convert(varchar(100),getdate(),23)+' 23:59:59' \n" +
                "                SELECT ISNULL(SUM(ISNULL(T.[sum],0)/100.0),0) order_sum,COUNT(1) order_count,\n" +
                "                ISNULL(SUM(CASE WHEN T.ShippingTime>T.CreationDate THEN 1 ELSE 0 END),0) ship_count\n" +
                "                FROM [Mall].[dbo].[Orders] T\n" +
                "                LEFT JOIN [Expo10_Passport]..Account T2 ON T2.Id=T.AccountId\n" +
                "                LEFT JOIN [Expo10_Passport]..Departments T3 ON T3.DeptCode=T.DeptCode\n" +
                "                WHERE T.DeptCode like '001608%'   AND T.OrderStatus not in(0,100,200) AND T.IsReturn=0\n" +
                "                AND T.OrderTime>=@d1 AND  T.OrderTime<=@d2");
        try {
            while (rs.next()) {
                csTotal.setOrder_sum(rs.getObject("order_sum").toString());
                csTotal.setOrder_count(rs.getObject("order_count").toString());
                csTotal.setShip_count(rs.getObject("ship_count").toString());
                return  csTotal;
            }
        } catch (SQLException e) {
            e.printStackTrace();

        }

        return csTotal;
    }

    public static List<TermOrder> GetOrderData() {
        ResultSet rs = SQLHelper.executeQuery("" +
                "USE Mall\n" +
                "DECLARE @d1 DATETIME,@d2 DATETIME\n" +
                "SET @d1=convert(varchar(100),getdate(),23)+' 00:00:00'\n" +
                "SET @d2=convert(varchar(100),getdate(),23)+' 23:59:59' \n" +
                "SET NOCOUNT ON \n" +
                "DECLARE @ta TABLE(order_sum MONEY,order_count INT,ship_count INT,term_type VARCHAR(50),term_class VARCHAR(50))\n" +
                "INSERT INTO @ta\n" +
                "( order_sum ,\n" +
                "order_count ,\n" +
                "ship_count ,\n" +
                "term_type ,\n" +
                "term_class\n" +
                ")\n" +
                "SELECT ISNULL(SUM(ISNULL(T.[sum],0)/100.0),0) order_sum,COUNT(1) order_count,\n" +
                "ISNULL(SUM(CASE WHEN T.ShippingTime>T.CreationDate THEN 1 ELSE 0 END),0) ship_count,'5一线' term_type,'广州运营团队' term_class\n" +
                "FROM [Mall].[dbo].[Orders] T WITH(NOLOCK)\n" +
                "LEFT JOIN [Expo10_Passport]..Account T2 ON T2.Id=T.AccountId\n" +
                "LEFT JOIN [Expo10_Passport]..Departments T3 ON T3.DeptCode=T.DeptCode\n" +
                "WHERE T.DeptCode like '000212%'   AND T.OrderStatus not in(0,100,200) AND T.IsReturn=0\n" +
                "AND T.OriginType IN (1,2,3,4,21,20,11,26,41,42,43,45,46,60,61,62,63)\n" +
                "AND T.OrderTime >=@d1 AND T.OrderTime <=@d2\n" +
                "UNION\n" +
                "SELECT ISNULL(SUM(ISNULL(T.[sum],0)/100.0),0) order_sum,COUNT(1) order_count,\n" +
                "ISNULL(SUM(CASE WHEN T.ShippingTime>T.CreationDate THEN 1 ELSE 0 END),0) ship_count,'6二线' term_type,'广州运营团队' term_class\n" +
                "FROM [Mall].[dbo].[Orders] T WITH(NOLOCK)\n" +
                "LEFT JOIN [Expo10_Passport]..Account T2 ON T2.Id=T.AccountId\n" +
                "LEFT JOIN [Expo10_Passport]..Departments T3 ON T3.DeptCode=T.DeptCode\n" +
                "WHERE T.DeptCode like '0003%'   AND T.OrderStatus not in(0,100,200) AND T.IsReturn=0\n" +
                "AND T.OriginType IN (1,2,3,4,21,20,11,26,41,42,43,45,46,60,61,62,63)\n" +
                "AND T.OrderTime >=@d1 AND T.OrderTime <=@d2\n" +
                "UNION\n" +
                "SELECT ISNULL(SUM(ISNULL(T.[sum],0)/100.0),0) order_sum,COUNT(1) order_count,\n" +
                "ISNULL(SUM(CASE WHEN T.ShippingTime>T.CreationDate THEN 1 ELSE 0 END),0) ship_count,'7第三方' term_type,'广州运营团队' term_class\n" +
                "FROM [Mall].[dbo].[Orders] T WITH(NOLOCK)\n" +
                "LEFT JOIN [Expo10_Passport]..Account T2 ON T2.Id=T.AccountId\n" +
                "LEFT JOIN [Expo10_Passport]..Departments T3 ON T3.DeptCode=T.DeptCode\n" +
                "WHERE (T.DeptCode like '0004%' OR T.DeptCode LIKE '000212%') AND T.OrderStatus not in(0,100,200) AND T.IsReturn=0\n" +
                "AND T.OriginType IN (5,6,7,8,9,10,12,14,15,17,18,19,22,23,24,25,29,27,28,31,32,33,34,35,36,37,38,39,40,44,47,48,49,50,51,52,53)\n" +
                "AND T.OrderTime >=@d1 AND T.OrderTime <=@d2\n" +
                "UNION\n" +
                "SELECT ISNULL(SUM(ISNULL(K.[Sum],0)/100.0),0) order_sum,COUNT(1) order_count,\n" +
                "ISNULL(SUM(CASE WHEN K.ShippingTime>K.CreationDate THEN 1 ELSE 0 END),0) ship_count,\n" +
                "K.term_type,'长沙运营团队' term_class\n" +
                "FROM\n" +
                "(\n" +
                "SELECT \n" +
                "T.Sum,\n" +
                "T.CreationDate,\n" +
                "T.ShippingTime,\n" +
                "CASE WHEN T.OriginType IN (6,40) THEN '1天猫'\n" +
                "WHEN T.OriginType=14 THEN '2京东'\n" +
                "WHEN T.OriginType=17 THEN '3平安好医生'\n" +
                "ELSE\n" +
                "'4苏宁'\n" +
                "END term_type\n" +
                "FROM dbo.Orders T WITH(NOLOCK)\n" +
                "WHERE T.OrderTime>=@d1 AND T.OrderTime<=@d2 \n" +
                "AND T.OriginType IN (6,40,14,17,50) AND T.OrderStatus NOT IN (0,100,200) AND T.IsReturn=0\n" +
                ")K\n" +
                "GROUP BY K.term_type\n" +
                "\n" +
                "\n" +
                "IF NOT EXISTS(SELECT 1 FROM @ta T WHERE T.term_type='1天猫')\n" +
                "INSERT INTO @ta\n" +
                "(order_sum ,\n" +
                "order_count ,\n" +
                "ship_count ,\n" +
                "term_type ,\n" +
                "term_class\n" +
                ")\n" +
                "VALUES  ( 0 , -- order_sum - varchar(50)\n" +
                "0 , -- order_count - int\n" +
                "0 , -- ship_count - int\n" +
                "'1天猫' , -- term_type - varchar(50)\n" +
                "'长沙运营团队'  -- term_class - varchar(50)\n" +
                ")\n" +
                "\n" +
                "IF NOT EXISTS(SELECT 1 FROM @ta T WHERE T.term_type='2京东')\n" +
                "INSERT INTO @ta\n" +
                "( order_sum ,\n" +
                "order_count ,\n" +
                "ship_count ,\n" +
                "term_type ,\n" +
                "term_class\n" +
                ")\n" +
                "VALUES  ( 0 , -- order_sum - varchar(50)\n" +
                "0 , -- order_count - int\n" +
                "0 , -- ship_count - int\n" +
                "'2京东' , -- term_type - varchar(50)\n" +
                "'长沙运营团队'  -- term_class - varchar(50)\n" +
                ")\n" +
                "\n" +
                "\n" +
                "IF NOT EXISTS(SELECT 1 FROM @ta T WHERE T.term_type='3平安好医生')\n" +
                "INSERT INTO @ta\n" +
                "( order_sum ,\n" +
                "order_count ,\n" +
                "ship_count ,\n" +
                "term_type ,\n" +
                "term_class\n" +
                ")\n" +
                "VALUES  ( 0 , -- order_sum - varchar(50)\n" +
                "0 , -- order_count - int\n" +
                "0 , -- ship_count - int\n" +
                "'3平安好医生' , -- term_type - varchar(50)\n" +
                "'长沙运营团队'  -- term_class - varchar(50)\n" +
                ")\n" +
                "\n" +
                "IF NOT EXISTS(SELECT 1 FROM @ta T WHERE T.term_type='4苏宁')\n" +
                "INSERT INTO @ta\n" +
                "( order_sum ,\n" +
                "order_count ,\n" +
                "ship_count ,\n" +
                "term_type ,\n" +
                "term_class\n" +
                ")\n" +
                "VALUES  ( 0 , -- order_sum - varchar(50)\n" +
                "0 , -- order_count - int\n" +
                "0 , -- ship_count - int\n" +
                "'4苏宁' , -- term_type - varchar(50)\n" +
                "'长沙运营团队'  -- term_class - varchar(50)\n" +
                ")\n" +
                "SELECT * FROM @ta ORDER BY term_class,term_type");

        List<TermOrder> list = new ArrayList<TermOrder>();
        ResultSetMetaData md = null; //获得结果集结构信息,元数据
        try {
            md = rs.getMetaData();
            int columnCount = md.getColumnCount();   //获得列数
            while (rs.next()) {
                TermOrder termOrder = new TermOrder();
                for (int i = 1; i <= columnCount; i++) {
                    termOrder.setOrder_sum(rs.getObject("order_sum").toString());
                    termOrder.setOrder_count(rs.getObject("order_count").toString());
                    termOrder.setShip_count(rs.getObject("ship_count").toString());
                    termOrder.setTerm_type(rs.getObject("term_type").toString());
                    termOrder.setTerm_class(rs.getObject("term_class").toString());
                }
                list.add(termOrder);

            }
        } catch (SQLException e) {
            e.printStackTrace();
        }

        return list;
    }

    public static List<OrderPhase> GetPhases() {
        ResultSet rs = SQLHelper.executeQuery("USE Mall\n" +
                "DECLARE @d1 DATETIME,@d2 DATETIME,@d3 NVARCHAR(50)\n" +
                "SET @d3=convert(varchar(100),getdate(),23)\n" +
                "SET @d1=convert(varchar(100),getdate(),23)+' 00:00:00'\n" +
                "SET @d2=convert(varchar(100),getdate(),23)+' 23:59:59'\n" +
                "SELECT K.date_phase,SUM(K.Amount/100.0) order_amount \n" +
                "FROM \n" +
                "(\n" +
                "SELECT T.Sum Amount,\n" +
                "CASE\n" +
                "WHEN T.OrderTime <=@d3+' 1:00:00' THEN 1\n" +
                "WHEN T.OrderTime <=@d3+' 2:00:00' THEN 2\n" +
                "WHEN T.OrderTime <=@d3+' 3:00:00' THEN 3\n" +
                "WHEN T.OrderTime <=@d3+' 4:00:00' THEN 4\n" +
                "WHEN T.OrderTime <=@d3+' 5:00:00' THEN 5\n" +
                "WHEN T.OrderTime <=@d3+' 6:00:00' THEN 6\n" +
                "WHEN T.OrderTime <=@d3+' 7:00:00' THEN 7\n" +
                "WHEN T.OrderTime <=@d3+' 8:00:00' THEN 8\n" +
                "WHEN T.OrderTime <=@d3+' 9:00:00' THEN 9\n" +
                "WHEN T.OrderTime <=@d3+' 10:00:00' THEN 10\n" +
                "WHEN T.OrderTime <=@d3+' 11:00:00' THEN 11\n" +
                "WHEN T.OrderTime <=@d3+' 12:00:00' THEN 12\n" +
                "WHEN T.OrderTime <=@d3+' 13:00:00' THEN 13\n" +
                "WHEN T.OrderTime <=@d3+' 14:00:00' THEN 14\n" +
                "WHEN T.OrderTime <=@d3+' 15:00:00' THEN 15\n" +
                "WHEN T.OrderTime <=@d3+' 16:00:00' THEN 16\n" +
                "WHEN T.OrderTime <=@d3+' 17:00:00' THEN 17\n" +
                "WHEN T.OrderTime <=@d3+' 18:00:00' THEN 18\n" +
                "WHEN T.OrderTime <=@d3+' 19:00:00' THEN 19\n" +
                "WHEN T.OrderTime <=@d3+' 20:00:00' THEN 20\n" +
                "WHEN T.OrderTime <=@d3+' 21:00:00' THEN 21\n" +
                "WHEN T.OrderTime <=@d3+' 22:00:00' THEN 22\n" +
                "WHEN T.OrderTime <=@d3+' 23:00:00' THEN 23\n" +
                "ELSE 24 END date_phase\n" +
                "FROM [Mall].[dbo].[Orders] T\n" +
                "WHERE T.DeptCode NOT LIKE '000213%' --长沙O2O客服部\n" +
                "AND T.DeptCode NOT LIKE '000218%' --O2O部\n" +
                "AND T.DeptCode NOT LIKE '001701%' --门店电商中心\n" +
                "AND T.OrderStatus not in(0,100,200) AND T.IsReturn=0\n" +
                "AND T.OrderTime >=@d1 AND T.OrderTime <=@d2\n" +
                ") K\n" +
                "GROUP BY K.date_phase\n" +
                "ORDER BY K.date_phase");

        List<OrderPhase> list = new ArrayList<OrderPhase>();
        ResultSetMetaData md = null; //获得结果集结构信息,元数据
        try {
            md = rs.getMetaData();
            int columnCount = md.getColumnCount();   //获得列数
            while (rs.next()) {
                OrderPhase orderPhase = new OrderPhase();
                for (int i = 1; i <= columnCount; i++) {
                    orderPhase.setOrder_amount(rs.getObject("order_amount").toString());
                    orderPhase.setDate_phase(rs.getObject("date_phase").toString());
                }
                list.add(orderPhase);

            }
        } catch (SQLException e) {
            e.printStackTrace();
        }

        return list;


    }

}