package com.lj.lucene;

import com.lj.others.CMSArticles;
import com.lj.others.Config;
import com.lj.others.DrugGuideArticles;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.DirectoryReader;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.IndexWriterConfig;
import org.apache.lucene.queryparser.classic.MultiFieldQueryParser;
import org.apache.lucene.queryparser.classic.ParseException;
import org.apache.lucene.queryparser.classic.QueryParser;
import org.apache.lucene.search.*;
import org.apache.lucene.search.highlight.*;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.util.BytesRef;

import java.io.IOException;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Created by john on 2018/6/26.
 */
public class CMSUtil_R {
    public static Directory DIRECTORY = null;
    public static Analyzer ANALYZER = null;
    public static IndexReader INDEXREADER = null;
    public static IndexSearcher INDEXSEARCHER = null;

    public CMSUtil_R(String IndexPath, Analyzer analyzer) {
        ANALYZER = analyzer;
        try {
            DIRECTORY = FSDirectory.open(Paths.get(IndexPath));
            INDEXREADER = DirectoryReader.open(DIRECTORY);
            INDEXSEARCHER = new IndexSearcher(INDEXREADER);
        } catch (IOException e) {
        }
    }

    public CMSArticles SearchCMS(List<String> fields,
                                 String _input,
                                 Map<String, String> sorts,
                                 Map<String, String> ranges,
                                 Boolean ishlight) {


        CMSArticles _CMSArticles = new CMSArticles();
        List<DrugGuideArticles> _list = new ArrayList<DrugGuideArticles>();
        //多条件查询神器
        BooleanQuery.Builder builder = new BooleanQuery.Builder();
        //多条件查询神器
        String[] fieldsTemp = new String[fields.size()];
        for (int i = 0; i < fields.size(); i++) {
            fieldsTemp[i] = fields.get(i);
        }
        //加载多域查询
        QueryParser parser = new MultiFieldQueryParser(fieldsTemp, ANALYZER);
        try {
            Query query = parser.parse(_input);
            builder.add(query, BooleanClause.Occur.MUST);
        } catch (ParseException e) {
            e.printStackTrace();
        }

        //段查询
        if (ranges.size() > 0) {
            for (String key : ranges.keySet()) {
                String _value = ranges.get(key);
                String[] _arrs = _value.split("-");
                TermRangeQuery query = new TermRangeQuery(key, new BytesRef(_arrs[0]), new BytesRef(_arrs[1]), true, true);
                builder.add(query, BooleanClause.Occur.MUST);
            }
        }

        //排序
        Sort sort = new Sort();
        if (sorts.size() > 0) {
            //Sort sort =new Sort(new SortField("isbn",SortField.Type.STRING,true));
            SortField[] _sfarr = new SortField[sorts.size()];
            int i = 0;
            for (String key : sorts.keySet()) {
                String _value = sorts.get(key);
                boolean _paixu = false;
                if (_value.toLowerCase().equals("desc"))
                    _paixu = true;
                SortField sortField = new SortField(key, SortField.Type.LONG, _paixu);
                _sfarr[i] = sortField;
                i++;
            }
            sort.setSort(_sfarr);
        }
        String error = "";
        try {
            TopDocs tds = null;
            if (sort.getSort().length > 0) {
                tds = INDEXSEARCHER.search(builder.build(), 5000, sort);
            } else {
                tds = INDEXSEARCHER.search(builder.build(), 5000);
            }

            //注意 此处把500条数据放在内存里。
            ScoreDoc[] sds = tds.scoreDocs;
            _CMSArticles.setTotal(tds.totalHits);

            if (tds.totalHits > 0) {
                for (int i = 0; i < tds.totalHits; i++) {
                    Document doc = INDEXSEARCHER.doc(sds[i].doc);
                    System.out.println(doc.get("id"));

                    String[] _columns = Config.getCMSFileds();
                    String[] _colunmsValues = new String[_columns.length];
                    for (int k = 0; k < _columns.length; k++) {
                        if (ishlight == true && fields.contains(_columns[k])) {
                            try {
                                //这里进行高亮处理
                                SimpleHTMLFormatter simpleHtmlFormatter = new SimpleHTMLFormatter("<B>", "</B>");
                                Highlighter highlighter = new Highlighter(simpleHtmlFormatter, new QueryScorer(builder.build()));
                                highlighter.setTextFragmenter(new SimpleFragmenter(500));
                                String _temp = highlighter.getBestFragment(ANALYZER, _columns[k], doc.get(_columns[k]));
                                if (_temp != null) {
                                    _colunmsValues[k] = _temp;
                                } else {
                                    _colunmsValues[k] = doc.get(_columns[k]);
                                }
                            } catch (InvalidTokenOffsetsException e) {
                                e.printStackTrace();
                            }
                        } else {
                            _colunmsValues[k] = doc.get(_columns[k]);
                        }
                    }
                    DrugGuideArticles drugGuideArticles = new DrugGuideArticles(_colunmsValues);
                    drugGuideArticles.setTitle_noTag(doc.get("title"));
                    drugGuideArticles.setSummary_notag(doc.get("summary"));
                    drugGuideArticles.setKeyword_notag(doc.get("keyword"));
                    drugGuideArticles.setSubtitle_notag(doc.get("subtitle"));
                    drugGuideArticles.setArticlecontent_notag(doc.get("articlecontent"));
                    _CMSArticles.ArticlesList.add(drugGuideArticles);

                }

            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        System.out.println("hello");
        return _CMSArticles;
    }
}
