package com.lj.lucene;
import com.lj.others.*;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.*;
import org.apache.lucene.queryparser.classic.MultiFieldQueryParser;
import org.apache.lucene.queryparser.classic.ParseException;
import org.apache.lucene.queryparser.classic.QueryParser;
import org.apache.lucene.search.*;
import org.apache.lucene.search.highlight.*;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.util.BytesRef;

import java.io.IOException;
import java.nio.file.Paths;
import java.sql.ResultSet;
import java.util.*;

/**
 * Created by john on 2018/5/30.
 */
public class IndexUtil {

    private static Directory DIRECTORY = null;
    private static Analyzer ANALYZER = null;
    private static IndexWriter INDEXWRITER = null;

    public IndexUtil(String IndexPath, Analyzer analyzer) {
        ANALYZER = analyzer;
        try {
            DIRECTORY = FSDirectory.open(Paths.get(IndexPath));
            INDEXWRITER = new IndexWriter(DIRECTORY, new IndexWriterConfig(ANALYZER));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * @param rs                  数据库源
     * @param not_analyzedColumns 不分词的字段
     * @param numericColumns      数字字段，主要用来排序和段筛选
     * @param pinyinColumns       支持拼音检索的字段
     * @return
     */
    public int CreateIndex(ResultSet rs, String[] not_analyzedColumns, String[] numericColumns, String[] pinyinColumns) {
        List<Document> documents = ResultSetToObject.ProductResultSetToDocuments(rs, not_analyzedColumns, numericColumns, pinyinColumns);
        for (Document doc : documents) {
            try {
                INDEXWRITER.addDocument(doc);

            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        try {
            INDEXWRITER.commit();
        } catch (IOException e) {
            e.printStackTrace();
        }

        if (INDEXWRITER != null) {
            try {
                INDEXWRITER.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        if (INDEXWRITER.getDirectory() != null) {
            try {
                INDEXWRITER.getDirectory().close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return documents.size();
    }

    /**
     * 更新一条索引
     *
     * @param field
     * @param value
     * @param doc
     */
    public void UpdateSingleIndex(String field, String value, Document doc) {
        try {
            INDEXWRITER.updateDocument(new Term(field, value), doc);
            INDEXWRITER.commit();
           /* INDEXWRITER.close();
            DIRECTORY.close();*/
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 删除单一索引
     *
     * @param
     * @param
     */
    public void DeleteSiginIndexByKey(String field, String value){
        try {
            INDEXWRITER.deleteDocuments(new Term(field, value));
            INDEXWRITER.commit();
        } catch (IOException e) {
            e.printStackTrace();
        }

    }


    /**
     * 新增加一个索引
     *
     * @param document
     */
    public void InsertSiginIndex(Document document) {
        try {
            INDEXWRITER.addDocument(document);
            INDEXWRITER.commit();
           /* INDEXWRITER.close();
            DIRECTORY.close();*/
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 删除目录下所有索引
     */
    public boolean DeleteAllIndex() {
        try {
            INDEXWRITER.deleteAll();
            INDEXWRITER.commit();
           /* INDEXWRITER.close();
            DIRECTORY.close();*/
            return true;
        } catch (IOException e) {
            e.printStackTrace();
            return false;
        }
    }


}